"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var dotenv_1 = __importDefault(require("dotenv"));
var path = __importStar(require("path"));
var plugins_1 = __importDefault(require("./plugins"));
var loaders_1 = __importDefault(require("./loaders"));
var deepmerge_1 = __importDefault(require("deepmerge"));
var configs_1 = __importDefault(require("./configs"));
var webpack_chain_1 = __importDefault(require("webpack-chain"));
var HashTypes_1 = require("./types/HashTypes");
var EnvironmentTypes_1 = require("./types/EnvironmentTypes");
var AbstractBundler = (function () {
    function AbstractBundler(mode, config) {
        if (mode === void 0) { mode = EnvironmentTypes_1.EnvironmentTypes.Development; }
        if (config === void 0) { config = {}; }
        this.webpackChain = new webpack_chain_1.default();
        this.bundles = [];
        this.setupEnv(mode);
        this.setupConfig(config, process.cwd());
        this.presets();
    }
    AbstractBundler.prototype.aliases = function (aliases) {
        this.config.webpack.aliases = aliases;
        return this;
    };
    AbstractBundler.prototype.aggressiveVendorSplitting = function () {
        this.config.aggressiveVendorSplitting = true;
        new configs_1.default.Optimization(this);
        return this;
    };
    AbstractBundler.prototype.build = function () {
        this.addBundle(this.bundle());
        if (this.env.isInspecting) {
            process.exit();
        }
        return this.bundles;
    };
    AbstractBundler.prototype.browserSync = function (options) {
        if (this.env.isHot) {
            this.config.webpack.devServer.open = false;
        }
        return this;
    };
    AbstractBundler.prototype.chainWebpack = function (callback) {
        callback(this.webpackChain, this.env);
        return this;
    };
    AbstractBundler.prototype.copy = function (from, to) {
        if (to === void 0) { to = ""; }
        this.config.plugins.copy.patterns.push({
            from: path.join(this.config.root, from),
            to: path.join(this.config.outputPath, to),
        });
        return this;
    };
    AbstractBundler.prototype.dontClean = function (exclude) {
        var _this = this;
        if (!Array.isArray(exclude)) {
            exclude = [exclude];
        }
        exclude.forEach(function (pattern) {
            _this.config.plugins.clean.excludeList.push(pattern);
        });
        new plugins_1.default.Clean(this, this.config.plugins.clean);
        return this;
    };
    AbstractBundler.prototype.eslint = function (options) {
        if (!this.env.isProduction) {
            new plugins_1.default.Eslint(this, options);
        }
        return this;
    };
    AbstractBundler.prototype.html = function () {
        new loaders_1.default.Html(this);
        return this;
    };
    AbstractBundler.prototype.htmlVariables = function (variables) {
        this.config.plugins.html.variables = Object.assign({}, this.config.plugins.html.variables, variables);
        new plugins_1.default.Html(this, this.config.plugins.html);
        return this;
    };
    AbstractBundler.prototype.plugin = function (Plugin, options) {
        new Plugin(this, options);
        return this;
    };
    AbstractBundler.prototype.proxy = function (from, to, options) {
        if (options === void 0) { options = {
            changeOrigin: true,
        }; }
        this.config.webpack.devServer.proxies.push(Object.assign({
            context: Array.isArray(from) ? from : [from],
            target: to,
        }, options));
        return this;
    };
    AbstractBundler.prototype.purgeCss = function (paths, config) {
        this.config.plugins.purgeCss = Object.assign({}, { paths: paths }, config);
        new plugins_1.default.PurgeCss(this, this.config.plugins.purgeCss);
        return this;
    };
    AbstractBundler.prototype.globalSassIncludes = function (filePaths, rx) {
        var _this = this;
        if (rx === void 0) { rx = /\.vue$/; }
        if (!Array.isArray(filePaths)) {
            filePaths = [filePaths];
        }
        this.config.loaders.sassLoader.globalIncludes = this.config.loaders.sassLoader.globalIncludes.concat(filePaths);
        this.webpackChain.module
            .rule("sass")
            .oneOf("vue")
            .use("sass-loader")
            .tap(function (options) {
            options.additionalData = function (content, loaderContext) {
                if (rx.test(loaderContext.resourcePath)) {
                    content =
                        _this.config.loaders.sassLoader.globalIncludes
                            .map(function (_filePath) {
                            return "@import \"" + path
                                .join(_this.config.root, _filePath)
                                .replace(/\\/g, "/") + "\";";
                        })
                            .join("\n") + content;
                }
                return content;
            };
            return options;
        });
        return this;
    };
    AbstractBundler.prototype.varieConfig = function (variables) {
        this.config.plugins.defineEnvironmentVariables.variables = variables;
        return this;
    };
    AbstractBundler.prototype.webWorkers = function () {
        new loaders_1.default.WebWorkers(this);
        return this;
    };
    AbstractBundler.prototype.addBundle = function (bundle) {
        if (this.env.isInspecting) {
            this.inspect(bundle);
        }
        this.bundles.push(bundle.toConfig());
    };
    AbstractBundler.prototype.argumentsHas = function (argument) {
        var commandLineArguments = process.argv;
        return commandLineArguments
            ? commandLineArguments.includes(argument)
            : false;
    };
    AbstractBundler.prototype.bundle = function () {
        var _this = this;
        this.webpackChain.when(!this.env.isProduction, function () {
            new plugins_1.default.WebpackBar(_this, {
                name: _this.config.bundleName,
            });
        });
        new configs_1.default.Aliases(this, this.config.webpack.aliases);
        new plugins_1.default.DefineEnvironmentVariables(this, this.config.plugins.defineEnvironmentVariables);
        if (this.config.plugins.copy.patterns.length > 0) {
            new plugins_1.default.Copy(this, this.config.plugins.copy);
        }
        this.webpackChain.when(this.env.isHot, function () {
            new configs_1.default.DevServer(_this, _this.config.webpack.devServer);
        });
        return this.webpackChain;
    };
    AbstractBundler.prototype.inspect = function (bundle) {
        var logName = this.config.bundleName + "-bundle-webpack-output.json";
        fs_1.default.writeFileSync(logName, bundle.toString());
        console.log("\n Wrote : " + this.config.bundleName + " Bundle to " + logName + "\n");
    };
    AbstractBundler.prototype.setupConfig = function (config, root) {
        if (config === void 0) { config = {}; }
        var envConfig = dotenv_1.default.config().parsed || {};
        var host = envConfig.APP_HOST || "0.0.0.0";
        var outputPath = path.join(root, envConfig.OUTPUT_PATH || "public");
        this.config = (0, deepmerge_1.default)({
            root: root,
            host: host,
            outputPath: outputPath,
            bundleName: "Client",
            aggressiveVendorSplitting: false,
            appName: envConfig.APP_NAME || "Varie",
            hashType: this.env.isHot ? HashTypes_1.HashTypes.Hash : HashTypes_1.HashTypes.ContentHash,
            cache: this.env.isHot && !this.argumentsHas("--disable-cache"),
            plugins: {
                copy: {
                    patterns: [],
                },
                html: {
                    variables: {},
                },
                browserSync: {
                    host: host,
                    outputPath: outputPath,
                    port: 3000,
                    proxy: host + ":8080",
                },
                clean: {
                    excludeList: [],
                },
                defineEnvironmentVariables: {
                    variables: [],
                },
            },
            loaders: {
                sassLoader: {
                    globalIncludes: [],
                },
            },
            webpack: {
                aliases: {},
                entryFiles: [],
                devServer: {
                    host: host,
                    open: true,
                    proxies: [],
                    port: 8080,
                },
            },
            vue: {
                runtimeOnly: true,
            },
        }, config);
    };
    AbstractBundler.prototype.setupEnv = function (mode) {
        this.env = {
            mode: mode,
            isHot: this.argumentsHas("--hot"),
            isAnalyzing: this.argumentsHas("--analyze"),
            isInspecting: this.argumentsHas("--inspect"),
            isProduction: mode === EnvironmentTypes_1.EnvironmentTypes.Production,
            isDevelopment: mode === EnvironmentTypes_1.EnvironmentTypes.Development,
        };
    };
    AbstractBundler.prototype.presets = function () {
        var _this = this;
        new plugins_1.default.CaseSensitivePaths(this);
        this.webpackChain.when(this.env.isAnalyzing, function () {
            new plugins_1.default.BundleAnalyzer(_this);
        });
        this.webpackChain
            .mode(this.env.mode)
            .context(this.config.root)
            .devtool(this.env.isProduction ? "hidden-source-map" : "eval-source-map")
            .resolve.symlinks(false);
        new configs_1.default.Stats(this);
        new configs_1.default.Output(this);
        new configs_1.default.Extensions(this);
        new configs_1.default.Optimization(this);
    };
    AbstractBundler.prototype.updateJavascriptLoaders = function () {
        new loaders_1.default.Javascript(this, {
            entryFiles: this.config.webpack.entryFiles,
        });
        new loaders_1.default.Typescript(this, {
            entryFiles: this.config.webpack.entryFiles,
        });
    };
    return AbstractBundler;
}());
exports.default = AbstractBundler;
//# sourceMappingURL=AbstractBundler.js.map